use crate::vulkan::vk_render_pass::VkRenderPass;
use crate::vulkan::{VkDevice, VkSwapchain};
use ash::prelude::VkResult;
use ash::vk;
use std::sync::Arc;

pub struct VkSwapchainFramebuffer {
    device: Arc<VkDevice>,
    swapchain: Arc<VkSwapchain>,
    render_pass: Arc<VkRenderPass>,
    image_view_index: usize,

    pub(super) handle: vk::Framebuffer,
}

impl VkSwapchainFramebuffer {
    pub fn new(
        device: Arc<VkDevice>,
        swapchain: Arc<VkSwapchain>,
        render_pass: Arc<VkRenderPass>,
        image_view_index: usize,
    ) -> VkResult<Self> {
        let present_image_view = swapchain
            .present_image_views
            .as_ref()
            .unwrap()[image_view_index];
        let attachments = [present_image_view];
        let framebuffer_info = vk::FramebufferCreateInfo::default()
            .render_pass(render_pass.handle)
            .width(swapchain.surface_resolution.width)
            .height(swapchain.surface_resolution.height)
            .attachments(&attachments)
            .layers(1);

        let framebuffer = unsafe { device.handle.create_framebuffer(&framebuffer_info, None)? };

        Ok(Self {
            device,
            swapchain,
            render_pass,
            image_view_index,

            handle: framebuffer,
        })
    }
}

impl Drop for VkSwapchainFramebuffer {
    fn drop(&mut self) {
        unsafe { self.device.handle.destroy_framebuffer(self.handle, None) };
    }
}